package javax.swing;

import java.text.Collator;
import java.util.Arrays;
import java.util.Iterator;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.Comparator;

public abstract class DefaultRowSorter<M, I> extends RowSorter<M>
{
    private boolean sortsOnUpdates;
    private DefaultRowSorter.Row[] viewToModel;
    private int[] modelToView;
    private Comparator[] comparators;
    private boolean[] isSortable;
    private RowSorter.SortKey[] cachedSortKeys;
    private Comparator[] sortComparators;
    private RowFilter<? super M, ? super I> filter;
    private DefaultRowSorter.FilterEntry filterEntry;
    private List<RowSorter.SortKey> sortKeys;
    private boolean[] useToString;
    private boolean sorted;
    private int maxSortKeys;
    private DefaultRowSorter.ModelWrapper<M, I> modelWrapper;
    private int modelRowCount;
    
    public DefaultRowSorter() {
        this.sortKeys = Collections.emptyList();
        this.maxSortKeys = 3;
    }
    
    protected final void setModelWrapper(DefaultRowSorter.ModelWrapper<M, I> modelWrapper) {
        if (modelWrapper == null) {
            throw new IllegalArgumentException("modelWrapper most be non-null");
        }
        DefaultRowSorter.ModelWrapper<Object, Object> modelWrapper2 = this.modelWrapper;
        this.modelWrapper = modelWrapper;
        if (modelWrapper2 != null) {
            this.modelStructureChanged();
        }
        else {
            this.modelRowCount = this.getModelWrapper().getRowCount();
        }
        return;
    }
    
    protected final DefaultRowSorter.ModelWrapper<M, I> getModelWrapper() {
        return this.modelWrapper;
    }
    
    @Override
    public final M getModel() {
        return this.getModelWrapper().getModel();
    }
    
    public void setSortable(int n, boolean b) {
        this.checkColumn(n);
        if (this.isSortable == null) {
            this.isSortable = new boolean[this.getModelWrapper().getColumnCount()];
            for (int i = this.isSortable.length - 1; i >= 0; --i) {
                this.isSortable[i] = true;
            }
        }
        this.isSortable[n] = b;
    }
    
    public boolean isSortable(int n) {
        this.checkColumn(n);
        return this.isSortable == null || this.isSortable[n];
    }
    
    @Override
    public void setSortKeys(List<? extends RowSorter.SortKey> c) {
        List<Object> sortKeys = this.sortKeys;
        if (c != null && c.size() > 0) {
            int columnCount = this.getModelWrapper().getColumnCount();
            for (RowSorter.SortKey sortKey : c) {
                if (sortKey == null || sortKey.getColumn() < 0 || sortKey.getColumn() >= columnCount) {
                    throw new IllegalArgumentException("Invalid SortKey");
                }
                continue;
            }
            this.sortKeys = Collections.unmodifiableList(new ArrayList(c));
        }
        else {
            this.sortKeys = Collections.emptyList();
        }
        if (!this.sortKeys.equals(sortKeys)) {
            this.fireSortOrderChanged();
            if (this.viewToModel == null) {
                this.sort();
            }
            else {
                this.sortExistingData();
            }
        }
    }
    
    @Override
    public List<? extends RowSorter.SortKey> getSortKeys() {
        return this.sortKeys;
    }
    
    public void setMaxSortKeys(int maxSortKeys) {
        if (maxSortKeys < 1) {
            throw new IllegalArgumentException("Invalid max");
        }
        this.maxSortKeys = maxSortKeys;
        return;
    }
    
    public int getMaxSortKeys() {
        return this.maxSortKeys;
    }
    
    public void setSortsOnUpdates(boolean sortsOnUpdates) {
        this.sortsOnUpdates = sortsOnUpdates;
    }
    
    public boolean getSortsOnUpdates() {
        return this.sortsOnUpdates;
    }
    
    public void setRowFilter(RowFilter<? super M, ? super I> filter) {
        this.filter = filter;
        this.sort();
    }
    
    public RowFilter<? super M, ? super I> getRowFilter() {
        return this.filter;
    }
    
    @Override
    public void toggleSortOrder(int n) {
        this.checkColumn(n);
        if (this.isSortable(n)) {
            ArrayList subList;
            int n2;
            for (subList = new ArrayList(this.getSortKeys()), n2 = subList.size() - 1; n2 >= 0 && ((RowSorter.SortKey)subList.get(n2)).getColumn() != n; --n2) {}
            if (n2 == -1) {
                subList.add(0, new RowSorter.SortKey(n, SortOrder.ASCENDING));
            }
            else if (n2 == 0) {
                subList.set(0, this.toggle((RowSorter.SortKey)subList.get(0)));
            }
            else {
                subList.remove(n2);
                subList.add(0, new RowSorter.SortKey(n, SortOrder.ASCENDING));
            }
            if (subList.size() > this.getMaxSortKeys()) {
                subList = subList.subList(0, this.getMaxSortKeys());
            }
            this.setSortKeys(subList);
        }
    }
    
    private RowSorter.SortKey toggle(RowSorter.SortKey sortKey) {
        if (sortKey.getSortOrder() == SortOrder.ASCENDING) {
            return new RowSorter.SortKey(sortKey.getColumn(), SortOrder.DESCENDING);
        }
        return new RowSorter.SortKey(sortKey.getColumn(), SortOrder.ASCENDING);
    }
    
    @Override
    public int convertRowIndexToView(int n) {
        if (this.modelToView == null) {
            if (n < 0 || n >= this.getModelWrapper().getRowCount()) {
                throw new IndexOutOfBoundsException("Invalid index");
            }
            return n;
        }
        else {
            return this.modelToView[n];
        }
    }
    
    @Override
    public int convertRowIndexToModel(int n) {
        if (this.viewToModel == null) {
            if (n < 0 || n >= this.getModelWrapper().getRowCount()) {
                throw new IndexOutOfBoundsException("Invalid index");
            }
            return n;
        }
        else {
            return this.viewToModel[n].modelIndex;
        }
    }
    
    private boolean isUnsorted() {
        List<Object> sortKeys = this.getSortKeys();
        return sortKeys.size() == 0 || ((RowSorter.SortKey)sortKeys.get(0)).getSortOrder() == SortOrder.UNSORTED;
    }
    
    private void sortExistingData() {
        int[] viewToModelAsInts = this.getViewToModelAsInts(this.viewToModel);
        this.updateUseToString();
        this.cacheSortKeys(this.getSortKeys());
        if (this.isUnsorted()) {
            if (this.getRowFilter() == null) {
                this.viewToModel = null;
                this.modelToView = null;
            }
            else {
                int n = 0;
                for (int i = 0; i < this.modelToView.length; ++i) {
                    if (this.modelToView[i] != -1) {
                        this.viewToModel[n].modelIndex = i;
                        this.modelToView[i] = n++;
                    }
                }
            }
        }
        else {
            Arrays.sort(this.viewToModel);
            this.setModelToViewFromViewToModel(false);
        }
        this.fireRowSorterChanged(viewToModelAsInts);
    }
    
    public void sort() {
        this.sorted = true;
        int[] viewToModelAsInts = this.getViewToModelAsInts(this.viewToModel);
        this.updateUseToString();
        if (this.isUnsorted()) {
            this.cachedSortKeys = new RowSorter.SortKey[0];
            if (this.getRowFilter() == null) {
                if (this.viewToModel != null) {
                    this.viewToModel = null;
                    this.modelToView = null;
                }
                else {
                    return;
                }
            }
            else {
                this.initializeFilteredMapping();
            }
        }
        else {
            this.cacheSortKeys(this.getSortKeys());
            if (this.getRowFilter() != null) {
                this.initializeFilteredMapping();
            }
            else {
                this.createModelToView(this.getModelWrapper().getRowCount());
                this.createViewToModel(this.getModelWrapper().getRowCount());
            }
            Arrays.sort(this.viewToModel);
            this.setModelToViewFromViewToModel(false);
        }
        this.fireRowSorterChanged(viewToModelAsInts);
    }
    
    private void updateUseToString() {
        int i = this.getModelWrapper().getColumnCount();
        if (this.useToString == null || this.useToString.length != i) {
            this.useToString = new boolean[i];
        }
        --i;
        while (i >= 0) {
            this.useToString[i] = this.useToString(i);
            --i;
        }
    }
    
    private void initializeFilteredMapping() {
        int rowCount = this.getModelWrapper().getRowCount();
        int n = 0;
        this.createModelToView(rowCount);
        for (int i = 0; i < rowCount; ++i) {
            if (this.include(i)) {
                this.modelToView[i] = i - n;
            }
            else {
                this.modelToView[i] = -1;
                ++n;
            }
        }
        this.createViewToModel(rowCount - n);
        int j = 0;
        int n2 = 0;
        while (j < rowCount) {
            if (this.modelToView[j] != -1) {
                this.viewToModel[n2++].modelIndex = j;
            }
            ++j;
        }
    }
    
    private void createModelToView(int n) {
        if (this.modelToView == null || this.modelToView.length != n) {
            this.modelToView = new int[n];
        }
    }
    
    private void createViewToModel(int a) {
        int i = 0;
        if (this.viewToModel != null) {
            i = Math.min(a, this.viewToModel.length);
            if (this.viewToModel.length != a) {
                System.arraycopy(this.viewToModel, 0, this.viewToModel = new DefaultRowSorter.Row[a], 0, i);
            }
        }
        else {
            this.viewToModel = new DefaultRowSorter.Row[a];
        }
        for (int j = 0; j < i; ++j) {
            this.viewToModel[j].modelIndex = j;
        }
        for (; i < a; ++i) {
            this.viewToModel[i] = new DefaultRowSorter.Row(this, i);
        }
    }
    
    private void cacheSortKeys(List<? extends RowSorter.SortKey> list) {
        int size = list.size();
        this.sortComparators = new Comparator[size];
        for (int i = 0; i < size; ++i) {
            this.sortComparators[i] = this.getComparator0(((RowSorter.SortKey)list.get(i)).getColumn());
        }
        this.cachedSortKeys = (RowSorter.SortKey[])list.toArray(new RowSorter.SortKey[size]);
    }
    
    protected boolean useToString(int n) {
        return this.getComparator(n) == null;
    }
    
    private void setModelToViewFromViewToModel(boolean b) {
        if (b) {
            for (int i = this.modelToView.length - 1; i >= 0; --i) {
                this.modelToView[i] = -1;
            }
        }
        for (int j = this.viewToModel.length - 1; j >= 0; --j) {
            this.modelToView[this.viewToModel[j].modelIndex] = j;
        }
    }
    
    private int[] getViewToModelAsInts(DefaultRowSorter.Row[] array) {
        if (array != null) {
            int[] array2 = new int[array.length];
            for (int i = array.length - 1; i >= 0; --i) {
                array2[i] = array[i].modelIndex;
            }
            return array2;
        }
        return new int[0];
    }
    
    public void setComparator(int n, Comparator<?> comparator) {
        this.checkColumn(n);
        if (this.comparators == null) {
            this.comparators = new Comparator[this.getModelWrapper().getColumnCount()];
        }
        this.comparators[n] = comparator;
    }
    
    public Comparator<?> getComparator(int n) {
        this.checkColumn(n);
        if (this.comparators != null) {
            return this.comparators[n];
        }
        return null;
    }
    
    private Comparator getComparator0(int n) {
        Comparator<Object> comparator = this.getComparator(n);
        if (comparator != null) {
            return comparator;
        }
        return Collator.getInstance();
    }
    
    private RowFilter.Entry<M, I> getFilterEntry(int modelIndex) {
        if (this.filterEntry == null) {
            this.filterEntry = new DefaultRowSorter.FilterEntry();
        }
        this.filterEntry.modelIndex = modelIndex;
        return this.filterEntry;
    }
    
    @Override
    public int getViewRowCount() {
        if (this.viewToModel != null) {
            return this.viewToModel.length;
        }
        return this.getModelWrapper().getRowCount();
    }
    
    @Override
    public int getModelRowCount() {
        return this.getModelWrapper().getRowCount();
    }
    
    private void allChanged() {
        this.modelToView = null;
        this.viewToModel = null;
        this.comparators = null;
        this.isSortable = null;
        if (this.isUnsorted()) {
            this.sort();
        }
        else {
            this.setSortKeys(null);
        }
    }
    
    @Override
    public void modelStructureChanged() {
        this.allChanged();
        this.modelRowCount = this.getModelWrapper().getRowCount();
    }
    
    @Override
    public void allRowsChanged() {
        this.modelRowCount = this.getModelWrapper().getRowCount();
        this.sort();
    }
    
    @Override
    public void rowsInserted(int n, int n2) {
        this.checkAgainstModel(n, n2);
        int rowCount = this.getModelWrapper().getRowCount();
        if (n2 >= rowCount) {
            throw new IndexOutOfBoundsException("Invalid range");
        }
        this.modelRowCount = rowCount;
        if (this.shouldOptimizeChange(n, n2)) {
            this.rowsInserted0(n, n2);
        }
        return;
    }
    
    @Override
    public void rowsDeleted(int n, int n2) {
        this.checkAgainstModel(n, n2);
        if (n >= this.modelRowCount || n2 >= this.modelRowCount) {
            throw new IndexOutOfBoundsException("Invalid range");
        }
        this.modelRowCount = this.getModelWrapper().getRowCount();
        if (this.shouldOptimizeChange(n, n2)) {
            this.rowsDeleted0(n, n2);
        }
        return;
    }
    
    @Override
    public void rowsUpdated(int n, int n2) {
        this.checkAgainstModel(n, n2);
        if (n >= this.modelRowCount || n2 >= this.modelRowCount) {
            throw new IndexOutOfBoundsException("Invalid range");
        }
        if (this.getSortsOnUpdates()) {
            if (this.shouldOptimizeChange(n, n2)) {
                this.rowsUpdated0(n, n2);
            }
        }
        else {
            this.sorted = false;
        }
        return;
    }
    
    @Override
    public void rowsUpdated(int n, int n2, int n3) {
        this.checkColumn(n3);
        this.rowsUpdated(n, n2);
    }
    
    private void checkAgainstModel(int n, int n2) {
        if (n > n2 || n < 0 || n2 < 0 || n > this.modelRowCount) {
            throw new IndexOutOfBoundsException("Invalid range");
        }
        return;
    }
    
    private boolean include(int n) {
        RowFilter<Object, Object> rowFilter = this.getRowFilter();
        return rowFilter == null || rowFilter.include(this.getFilterEntry(n));
    }
    
    private int compare(int n, int n2) {
        int i = 0;
        while (i < this.cachedSortKeys.length) {
            int column = this.cachedSortKeys[i].getColumn();
            SortOrder sortOrder = this.cachedSortKeys[i].getSortOrder();
            int compare;
            if (sortOrder == SortOrder.UNSORTED) {
                compare = n - n2;
            }
            else {
                String s;
                String s2;
                if (this.useToString[column]) {
                    s = this.getModelWrapper().getStringValueAt(n, column);
                    s2 = this.getModelWrapper().getStringValueAt(n2, column);
                }
                else {
                    s = this.getModelWrapper().getValueAt(n, column);
                    s2 = this.getModelWrapper().getValueAt(n2, column);
                }
                if (s == null) {
                    if (s2 == null) {
                        compare = 0;
                    }
                    else {
                        compare = -1;
                    }
                }
                else if (s2 == null) {
                    compare = 1;
                }
                else {
                    compare = this.sortComparators[i].compare(s, s2);
                }
                if (sortOrder == SortOrder.DESCENDING) {
                    compare *= -1;
                }
            }
            if (compare != 0) {
                return compare;
            }
            ++i;
        }
        return n - n2;
    }
    
    private boolean isTransformed() {
        return this.viewToModel != null;
    }
    
    private void insertInOrder(List<DefaultRowSorter.Row> list, DefaultRowSorter.Row[] a) {
        int n = 0;
        int size = list.size();
        int binarySearch;
        for (int i = 0; i < size; this.viewToModel[binarySearch + i] = (DefaultRowSorter.Row)list.get(i), n = binarySearch, ++i) {
            binarySearch = Arrays.binarySearch(a, list.get(i));
            if (binarySearch < 0) {
                binarySearch = -1 - binarySearch;
            }
            System.arraycopy(a, n, this.viewToModel, n + i, binarySearch - n);
        }
        System.arraycopy(a, n, this.viewToModel, n + size, a.length - n);
    }
    
    private boolean shouldOptimizeChange(int n, int n2) {
        if (!this.isTransformed()) {
            return false;
        }
        if (!this.sorted || n2 - n > this.viewToModel.length / 10) {
            this.sort();
            return false;
        }
        return true;
    }
    
    private void rowsInserted0(int i, int n) {
        int[] viewToModelAsInts = this.getViewToModelAsInts(this.viewToModel);
        int initialCapacity = n - i + 1;
        ArrayList list = new ArrayList(initialCapacity);
        for (; i <= n; ++i) {
            if (this.include(i)) {
                list.add(new DefaultRowSorter.Row(this, i));
            }
        }
        for (int j = this.modelToView.length - 1; j >= i; --j) {
            int n2 = this.modelToView[j];
            if (n2 != -1) {
                DefaultRowSorter.Row row = this.viewToModel[n2];
                row.modelIndex += initialCapacity;
            }
        }
        if (list.size() > 0) {
            Collections.sort(list);
            DefaultRowSorter.Row[] viewToModel = this.viewToModel;
            this.viewToModel = new DefaultRowSorter.Row[this.viewToModel.length + list.size()];
            this.insertInOrder(list, viewToModel);
        }
        this.createModelToView(this.getModelWrapper().getRowCount());
        this.setModelToViewFromViewToModel(true);
        this.fireRowSorterChanged(viewToModelAsInts);
    }
    
    private void rowsDeleted0(int i, int n) {
        int[] viewToModelAsInts = this.getViewToModelAsInts(this.viewToModel);
        int n2 = 0;
        int n3;
        for (; i <= n; ++i) {
            n3 = this.modelToView[i];
            if (n3 != -1) {
                ++n2;
                this.viewToModel[n3] = null;
            }
        }
        int n4 = n - i + 1;
        for (int j = this.modelToView.length - 1; j > n; --j) {
            int n5 = this.modelToView[j];
            if (n5 != -1) {
                DefaultRowSorter.Row row = this.viewToModel[n5];
                row.modelIndex -= n4;
            }
        }
        if (n2 > 0) {
            DefaultRowSorter.Row[] viewToModel = new DefaultRowSorter.Row[this.viewToModel.length - n2];
            int n6 = 0;
            int n7 = 0;
            for (int k = 0; k < this.viewToModel.length; ++k) {
                if (this.viewToModel[k] == null) {
                    System.arraycopy(this.viewToModel, n7, viewToModel, n6, k - n7);
                    n6 += k - n7;
                    n7 = k + 1;
                }
            }
            System.arraycopy(this.viewToModel, n7, viewToModel, n6, this.viewToModel.length - n7);
            this.viewToModel = viewToModel;
        }
        this.createModelToView(this.getModelWrapper().getRowCount());
        this.setModelToViewFromViewToModel(true);
        this.fireRowSorterChanged(viewToModelAsInts);
    }
    
    private void rowsUpdated0(int i, int n) {
        int[] viewToModelAsInts = this.getViewToModelAsInts(this.viewToModel);
        int initialCapacity = n - i + 1;
        if (this.getRowFilter() == null) {
            DefaultRowSorter.Row[] array2 = new DefaultRowSorter.Row[initialCapacity];
            int n6;
            for (n6 = 0, ; i <= n; ++i, ++n6) {
                array2[n6] = this.viewToModel[this.modelToView[i]];
            }
            Arrays.sort(array2);
            DefaultRowSorter.Row[] array3 = new DefaultRowSorter.Row[this.viewToModel.length - initialCapacity];
            int k = 0;
            int n7 = 0;
            while (k < this.viewToModel.length) {
                int modelIndex = this.viewToModel[k].modelIndex;
                if (modelIndex < i || modelIndex > n) {
                    array3[n7++] = this.viewToModel[k];
                }
                ++k;
            }
            this.insertInOrder(Arrays.asList(array2), array3);
            this.setModelToViewFromViewToModel(false);
        }
        else {
            ArrayList list = new ArrayList(initialCapacity);
            int n2 = 0;
            int n3 = 0;
            int n4 = 0;
            for (; i <= n; ++i) {
                if (this.modelToView[i] == -1) {
                    if (this.include(i)) {
                        list.add(new DefaultRowSorter.Row(this, i));
                        ++n2;
                    }
                }
                else {
                    if (!this.include(i)) {
                        ++n3;
                    }
                    else {
                        list.add(this.viewToModel[this.modelToView[i]]);
                    }
                    this.modelToView[i] = -2;
                    ++n4;
                }
            }
            Collections.sort(list);
            DefaultRowSorter.Row[] array = new DefaultRowSorter.Row[this.viewToModel.length - n4];
            int j = 0;
            int n5 = 0;
            while (j < this.viewToModel.length) {
                if (this.modelToView[this.viewToModel[j].modelIndex] != -2) {
                    array[n5++] = this.viewToModel[j];
                }
                ++j;
            }
            if (n2 != n3) {
                this.viewToModel = new DefaultRowSorter.Row[this.viewToModel.length + n2 - n3];
            }
            this.insertInOrder(list, array);
            this.setModelToViewFromViewToModel(true);
        }
        this.fireRowSorterChanged(viewToModelAsInts);
    }
    
    private void checkColumn(int n) {
        if (n < 0 || n >= this.getModelWrapper().getColumnCount()) {
            throw new IndexOutOfBoundsException("column beyond range of TableModel");
        }
        return;
    }
    
    static int access$100(DefaultRowSorter defaultRowSorter, int n, int n2) {
        return defaultRowSorter.compare(n, n2);
    }
    
    protected abstract static class ModelWrapper<M, I>
    {
        public abstract M getModel();
        
        public abstract int getColumnCount();
        
        public abstract int getRowCount();
        
        public abstract Object getValueAt(int p0, int p1);
        
        public String getStringValueAt(int n, int n2) {
            Object value = this.getValueAt(n, n2);
            if (value == null) {
                return "";
            }
            String string = value.toString();
            if (string == null) {
                return "";
            }
            return string;
        }
        
        public abstract I getIdentifier(int p0);
    }
    
    private class FilterEntry extends RowFilter.Entry<M, I>
    {
        int modelIndex;
        
        @Override
        public M getModel() {
            return DefaultRowSorter.this.getModelWrapper().getModel();
        }
        
        @Override
        public int getValueCount() {
            return DefaultRowSorter.this.getModelWrapper().getColumnCount();
        }
        
        @Override
        public Object getValue(int n) {
            return DefaultRowSorter.this.getModelWrapper().getValueAt(this.modelIndex, n);
        }
        
        @Override
        public String getStringValue(int n) {
            return DefaultRowSorter.this.getModelWrapper().getStringValueAt(this.modelIndex, n);
        }
        
        @Override
        public I getIdentifier() {
            return DefaultRowSorter.this.getModelWrapper().getIdentifier(this.modelIndex);
        }
    }
    
    private static class Row implements Comparable<DefaultRowSorter.Row>
    {
        private DefaultRowSorter sorter;
        int modelIndex;
        
        public Row(DefaultRowSorter sorter, int modelIndex) {
            this.sorter = sorter;
            this.modelIndex = modelIndex;
        }
        
        @Override
        public int compareTo(DefaultRowSorter.Row row) {
            return DefaultRowSorter.access$100(this.sorter, this.modelIndex, row.modelIndex);
        }
    }
}
